@@ -1,5 +1,6 @@
 from enum import Enum
-from typing import List, Optional
+
+from typing import List, Optional, Tuple
 
 from cortex_xdr_client.api.base_api import BaseAPI
 from cortex_xdr_client.api.models.alerts import (
@@ -14,7 +15,7 @@ from cortex_xdr_client.api.models.filters import (
 
 
 class AlertsAPI(BaseAPI):
-    def __init__(self, api_key_id: int, api_key: str, fqdn: str, timeout: tuple[int, int]) -> None:
+    def __init__(self, api_key_id: int, api_key: str, fqdn: str, timeout: Tuple[int, int]) -> None:
         super(AlertsAPI, self).__init__(api_key_id, api_key, fqdn, "alerts", timeout)
 
     # https://docs.paloaltonetworks.com/cortex/cortex-xdr/cortex-xdr-api/cortex-xdr-apis/incident-management/get-alerts.html
@@ -28,6 +29,18 @@ class AlertsAPI(BaseAPI):
                    search_from: int = None,
                    search_to: int = None,
                    ) -> Optional[GetAlertsResponse]:
+        """
+        Get a list of alerts with multiple events.
+
+        :param alert_id_list: List of integers of the Alert ID
+        :param alert_source_list: List of strings of the Alert source
+        :param severities: List of strings of the Alert severity
+        :param creation_time: Timestamp of the Creation time
+        :param after_creation: If the creation date will be the upper or lower bound limit.
+        :param search_to: Integer representing the end offset within the result set after which you do not want incidents returned.
+        :param search_from: Integer representing the starting offset within the query result set from which you want incidents returned.
+        :return: Returns a GetAlertsResponse object if successful.
+        """
         filters = []
 
         if alert_id_list is not None:
@@ -46,9 +59,7 @@ class AlertsAPI(BaseAPI):
 
         response = self._call(call_name="get_alerts_multi_events",
                               json_value=request_data)
-        if response.ok:
-            return GetAlertsResponse.parse_obj(response.json())
-        return None
+        return GetAlertsResponse.parse_obj(response.json())
 
 
 def get_enum_values(p: List[Enum]) -> List[str]:
